package cn.com.qmth.stmms.ms.admin.api;

import cn.com.qmth.stmms.ms.admin.assembler.WorkOverviewAssembler;
import cn.com.qmth.stmms.ms.admin.dto.WorkOverview;
import cn.com.qmth.stmms.ms.admin.service.WorkService;
import cn.com.qmth.stmms.ms.commons.utils.RandomUtil;
import cn.com.qmth.stmms.ms.core.domain.Level;
import cn.com.qmth.stmms.ms.core.domain.Work;
import cn.com.qmth.stmms.ms.core.repository.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Sort;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 评卷工作api
 * Created by zhengmin on 2016/10/14.
 */
@RestController
@RequestMapping("api/admin/works")
public class WorkApi {

    @Autowired
    private WorkRepo workRepo;

    @Autowired
    private LevelRepo levelRepo;

    @Autowired
    private MarkUserRepo markUserRepo;

    @Autowired
    private ExamQuestionRepo questionRepo;

    @Autowired
    private WorkService workService;

    @Autowired
    private StudentRepo studentRepo;

    @Autowired
    private WorkOverviewAssembler workOverviewAssembler;

    @Autowired
    RandomUtil randomUtil;

    /**
     * 列表
     *
     * @return
     */
    @RequestMapping(method = RequestMethod.GET)
    public List<Work> list() {
        Sort sort = new Sort(Sort.Direction.DESC, "id");
        return workRepo.findAll(sort);
    }

    /**
     * 单个信息
     *
     * @param work
     * @return
     */
    @RequestMapping(value = "{work}", method = RequestMethod.GET)
    public Work getOne(@PathVariable Work work) {
        if (work.isActive()) {
            randomUtil.getRandom(work.getId(), false);
        }
        return work;
    }

    /**
     * 新增
     *
     * @param work
     */
    @RequestMapping(method = RequestMethod.POST)
    public void create(@RequestBody Work work) {
        workService.save(work);
    }

    /**
     * 修改
     *
     * @param workId
     * @param work
     * @return
     */
    @RequestMapping(value = "{workId}", method = RequestMethod.PUT)
    public Work update(@PathVariable Long workId, @RequestBody Work work) {
        return workRepo.save(work);
    }

    /**
     * 激活为当前，采集和评卷用户登录影响
     *
     * @param workId
     */
    @RequestMapping(value = "{workId}", method = RequestMethod.PATCH)
    public void active(@PathVariable Long workId) {
        List<Work> works = workRepo.findAll();
        for (Work w : works) {
            w.setActive(false);
            if (w.getId().equals(workId)) {
                w.setActive(true);
                randomUtil.getRandom(w.getId(), false);
            }
        }
        workRepo.save(works);
    }

    /**
     * 删除
     *
     * @param workId
     */
    @RequestMapping(value = "{workId}", method = RequestMethod.DELETE)
    public void remove(@PathVariable Long workId) {
        workService.delete(workId);
    }

    //////////////////////    levels

    /**
     * 档位设定列表
     *
     * @param workId
     * @return
     */
    @RequestMapping(value = "{workId}/levels", method = RequestMethod.GET)
    public List<Level> list(@PathVariable Long workId) {
        return levelRepo.findByWorkId(workId);
    }

    /**
     * 删除档位
     *
     * @param workId
     * @param levelId
     */
    @RequestMapping(value = "{workId}/levels/{levelId}", method = RequestMethod.DELETE)
    public void deleteLevel(@PathVariable Long workId, @PathVariable Long levelId) {
        levelRepo.delete(levelId);
    }
    //////////////////////   overview

    /**
     * 总览，各统计数据
     *
     * @param work
     * @return
     */
    @RequestMapping(value = "{work}/overview", method = RequestMethod.GET)
    public WorkOverview overview(@PathVariable Work work) {
        return workOverviewAssembler.toDTO(work);
    }

}
